///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004 Intel Corporation 
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met: 
//
// * Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// * Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
// * Neither name of Intel Corporation nor the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

//
// FFTFilter.cpp: demonstrate filtering with FFT
//

#include "stdafx.h"
#include "ipp.h"
#include "tools.h"

void myMakeSrc_32f(Ipp32f** pSig, int len)
{
	*pSig = ippsMalloc_32f(len);

	ippsVectorJaehne_32f( *pSig, len, 1 );
}

void myMakeDst_32f(Ipp32f** pSig, int len)
{
	*pSig = ippsMalloc_32f(len);

	ippsZero_32f( *pSig, len );
}

// call spView on a complex signal's magnitude
void myShowCCS_32fc(Ipp32fc* pSrc, int len)
{
	Ipp32fc *pDst = ippsMalloc_32fc(len);
	Ipp32f *pDstMag = ippsMalloc_32f(len);

	ippsMagnitude_32fc(pSrc, pDstMag, len/2+1);
	spView_32f(pDstMag, len/2+1, "Unexpanded Filter Magnitude", 2);

	ippsConjCcs_32fc((Ipp32f*)pSrc, pDst, len);
	ippsMagnitude_32fc(pDst, pDstMag, len);
	spView_32f(pDstMag, len, "Filter Magnitude", 2);

	ippsFree(pDst);
	ippsFree(pDstMag);
}

// Make a filter that screens 1/2 of the spectrum
void myMakeCCSFilter_32fc(Ipp32fc** pFilter, int len)
{
	*pFilter = (Ipp32fc*)ippsMalloc_32f(len+2);

	Ipp32fc one = {1.0, 0.0};
	ippsZero_32fc( *pFilter, len/2+1 );
	ippsSet_32fc(one, *pFilter, len/4);
}

void myFilter_32f(Ipp32f* pSrc, Ipp32fc* pFilterCCS, Ipp32f*pDst, int order)
{
	IppsFFTSpec_R_32f *pFFTSpec;

	Ipp32fc *pTmp = (Ipp32fc*)ippsMalloc_32f((1<<order)+2);
	ippsFFTInitAlloc_R_32f( &pFFTSpec, order, IPP_FFT_DIV_INV_BY_N, ippAlgHintFast );
	ippsFFTFwd_RToCCS_32f(pSrc, (Ipp32f*)pTmp, pFFTSpec, 0 );

	ippsMul_32fc_I(pFilterCCS, pTmp, (1<<(order-1)) + 1);

	ippsFFTInv_CCSToR_32f((Ipp32f*)pTmp, pDst, pFFTSpec, 0 );
	ippsFree(pFFTSpec);
}

int main(int argc, char* argv[])
{
	int i,order;
	Ipp32f *pSrc, *pDst;
	Ipp32fc *pFilter;

	// Set drawing settings to distinguish the two signals
	DrawingSettings settings = 
	{
		1,1,
		PS_SOLID, PS_SOLID,
		0x00808080, 0x00000000,
		1,1,
		0x00cfcfcf, 0x007f7f7f
	};

	for (i=64,order=6; i<= 1024; i*=2,order++)
	{
		myMakeSrc_32f(&pSrc,i);
		myMakeDst_32f((Ipp32f**)&pDst,i*2);
		myMakeCCSFilter_32fc(&pFilter,i);
		myFilter_32f(pSrc, pFilter, pDst, order);
		myShowCCS_32fc(pFilter, 1<<order);
		spView2_32f( pSrc, i, pDst,i,"G: Source; B: Dest", 1, settings );
		ippsFree(pFilter);
		ippsFree(pDst);
		ippsFree(pSrc);
	}


	return 0;
}
